"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Sentry = require("@sentry/browser");
var types_1 = require("@sentry/types");
var utils_1 = require("@sentry/utils");
var http = require("tns-core-modules/http");
var platform_1 = require("tns-core-modules/platform/platform");
var app = require("tns-core-modules/application");
var trace = require("tns-core-modules/trace");
var enums_1 = require("tns-core-modules/ui/enums");
var page_1 = require("tns-core-modules/ui/page");
var transports_1 = require("@sentry/browser/dist/transports");
var page = require("tns-core-modules/ui/page").Page; // Needed for global events
var appversion = require("nativescript-appversion");
var orientation = require('nativescript-orientation');
require("nativescript-globalevents");
/**
 * NativeScript specific transport
 */
var NativeScriptTransport = /** @class */ (function (_super) {
    __extends(NativeScriptTransport, _super);
    function NativeScriptTransport() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Locks transport after receiving 429 response
        _this._disabledUntil = new Date(Date.now());
        return _this;
    }
    NativeScriptTransport.prototype.sendEvent = function (event) {
        var _this = this;
        if (new Date(Date.now()) < this._disabledUntil) {
            return Promise.reject({
                event: event,
                reason: "Transport locked till " + this._disabledUntil + " due to too many requests.",
                status: 429,
            });
        }
        return this._buffer.add(new utils_1.SyncPromise(function (resolve, reject) {
            http.request({
                method: "POST",
                headers: _this.options.headers,
                url: _this.url,
                timeout: 2000,
                content: JSON.stringify(event)
            }).then(function (response) {
                if (response.statusCode === 200) {
                    resolve();
                    return;
                }
                if (status === types_1.Status.RateLimit) {
                    var now = Date.now();
                    var retryAfter = response.headers['Retry-After'] + '';
                    _this._disabledUntil = new Date(now + utils_1.parseRetryAfterHeader(now, retryAfter));
                    utils_1.logger.warn("Too many requests, backing off till: " + _this._disabledUntil);
                }
                reject(response);
            })
                .catch(function (err) {
                var msg = "Sentry Transport Error: " + err;
                utils_1.logger.error(msg);
                reject(err);
            });
        }));
    };
    return NativeScriptTransport;
}(transports_1.BaseTransport));
var TraceSentry = /** @class */ (function () {
    function TraceSentry(dsn, environment, appVersion, release, enableAppBreadcrumbs) {
        if (environment === void 0) { environment = "debug"; }
        if (appVersion === void 0) { appVersion = ""; }
        if (release === void 0) { release = ""; }
        if (enableAppBreadcrumbs === void 0) { enableAppBreadcrumbs = true; }
        if (dsn === undefined || dsn === "") {
            throw new Error("Sentry DSN string required to configure Sentry TraceWriter");
        }
        this.initSentry(dsn, environment, appVersion, release, enableAppBreadcrumbs);
    }
    TraceSentry.prototype.write = function (message, category, type) {
        if (typeof (Sentry) === "undefined")
            return; // Do not process if Sentry plugin not loaded
        // Sentry only recognizes 'info', 'warning' and 'error' ('error' is default)
        var level = Sentry.Severity.Error;
        if (type === trace.messageType.log || type === trace.messageType.info) {
            level = Sentry.Severity.Info;
        }
        else if (type === trace.messageType.warn) {
            level = Sentry.Severity.Warning;
        }
        // Add category as a tag for log
        Sentry.setTag('trace_category', category);
        Sentry.captureMessage(message, level);
    };
    TraceSentry.prototype.initSentry = function (dsn, environment, appVersion, release, enableAppBreadcrumbs) {
        Sentry
            .init({
            dsn: dsn,
            environment: environment,
            release: release,
            transport: NativeScriptTransport,
        });
        Sentry.setTags({
            serverName: platform_1.device.uuid,
            device_type: platform_1.device.deviceType,
            device_lang: platform_1.device.language,
            device_family: platform_1.device.manufacturer,
            device_model: platform_1.device.model,
            device_orientation: enums_1.DeviceOrientation[orientation.getOrientation()],
            device_battery_level: '' + this.batteryPercent,
            os_name: platform_1.device.os,
            os_version: platform_1.device.osVersion,
            runtime_name: 'nativescript',
            runtime_version: global.__runtimeVersion
        });
        if (enableAppBreadcrumbs) {
            this.initAutoCrumbs();
        }
        if (!appVersion) {
            this.initAppVersion();
        }
        this.initBatteryStatus();
    };
    TraceSentry.prototype.initAutoCrumbs = function () {
        // Loaded
        page.on(page_1.Page.loadedEvent, function (args) {
            var p = args.object;
            Sentry.addBreadcrumb({
                message: "Page loaded",
                category: "debug",
                data: {
                    binding_context: p.bindingContext
                },
                level: Sentry.Severity.Info
            });
        });
        // NavigatedTo
        page.on(page_1.Page.navigatedToEvent, function (args) {
            var p = args.object;
            Sentry.addBreadcrumb({
                message: "App navigated to new page",
                category: "navigation",
                data: {
                    binding_context: p.bindingContext,
                    nav_context: p.navigationContext
                },
                level: Sentry.Severity.Info
            });
        });
        //Shown Modally
        page.on(page_1.Page.shownModallyEvent, function (args) {
            var p = args.object;
            Sentry.addBreadcrumb({
                message: "Page shown modally",
                category: "navigation",
                data: {
                    binding_context: p.bindingContext,
                    nav_context: args.context
                },
                level: Sentry.Severity.Info
            });
        });
    };
    TraceSentry.prototype.initAppVersion = function () {
        // Add app version tag (async)
        appversion.getVersionName()
            .then(function (version) {
            Sentry.setTag('app_version', version);
        });
    };
    TraceSentry.prototype.initBatteryStatus = function () {
        var _this = this;
        if (platform_1.isAndroid) {
            app.android.registerBroadcastReceiver(android.content.Intent.ACTION_BATTERY_CHANGED, function (context, intent) {
                var level = intent.getIntExtra(android.os.BatteryManager.EXTRA_LEVEL, -1);
                var scale = intent.getIntExtra(android.os.BatteryManager.EXTRA_SCALE, -1);
                _this.batteryPercent = (level / scale) * 100.0;
            });
        }
        else {
            app.ios.addNotificationObserver(UIDeviceBatteryLevelDidChangeNotification, function (notification) {
                _this.batteryPercent = UIDevice.currentDevice.batteryLevel * 100;
            });
        }
    };
    return TraceSentry;
}());
exports.TraceSentry = TraceSentry;
